#++++++++++++++++
#.IDENTIFICATION edb1.awk
#.LANGUAGE       gawk
#.AUTHOR         Francois Ochsenbein [CDS]
#.VERSION  1.0   11-Jan-1992
#.VERSION  1.1   06-Apr-1992: rm option
#.VERSION  1.2   30-May-1992: Date was not always edited --- bug fixed.
#.VERSION  1.3   09-Jun-1992: Had same name for Array & Variable ... Fixed
#.VERSION  1.4   05-Jan-1993: Added %F
#.VERSION  1.5   27-Jul-1993: Added function clearcat()
#.VERSION  1.6   08-Jul-1994: %J= removed
#.VERSION  1.7   26-Sep-1994: %j edited as BibCode
#.VERSION  1.8   11-Sep-1995: %U %t
#.VERSION  1.9   22-Jun-2000: %b = Bulletin
#.VERSION  2.0   31-Aug-2009: %p (popu) %id (yCat)
#.VERSION  2.1   05-Jun-2013: %g (sources)
#.VERSION  2.2   21-Nov-2014: %q (cite)
#.PURPOSE        Edit Catalogue Information
#.REMARKS	 Parameter "width" => Line Width
#----------------

function amp(s) {	# Replace + by &
    _tr_s = s;
    gsub(/[+]/, "\\&", _tr_s);
    return(_tr_s);
}
function last_name(str) {
    #print "#last_name(" str ")"
    n = split(str, a); lname=a[1];
    for(i=2; i<=n; i++) { 
      if(index(a[i],".")== 0) lname = lname " " a[i];
      else break;
    }
      return(lname)
}
BEGIN {
  blanks = "                                           ";
  if (width <= 1) width = 80;
  for (i=1; i <= width; i++) sepl = sepl "=";
  title["%A"] = "Authors:";
  title["%B"] = "Abstract:";
  title["%b"] = "Announced:";
  title["%C"] = "Contents:";
  title["%D"] = "Date:";
  title["%F"] = "Files:";
  title["%J"] = "Ref:";
  title["%J="] = "BibCode:";
  title["%R"] = "BibCode:";
  title["%K"] = "Keyword:";
  title["%M"] = "Media:";
  title["%N"] = "Remark:";
  title["%p"] = "Popular:";
  title["%W"] = "cServer:";
  title["%Q"] = "qProgram:";
  title["%c"] = "Category:";
  title["%w-"] = "ignored:";
  title["%w+"] = "inVizieR:";
  title["%g"] = "Sources:";
  title["%q"] = "Cite:";
  #if (index(rm, "o") != 0)  rmo = 1; else rmo = 0;
}
##
# Save a line
##
function additem(head, body){ # Add a new part
	if (I == "") next;
	tp = head; if (head == field) tp = "";
	if (tp in title) tp = title[tp];
	nL++;
	aT[nL] = tp;
	aL[nL] = body;
	field = head;
	next;
}
#
#	Output a line --- limited to 80 bytes ---
#
function oline(t, s) {
	# print "Arguments t=" t "s=" s;
	if (length(t) < 9) { len = 9;
		line = line t substr(blanks, 1, 9-length(t));
#		printf("%s%s", t, substr(blanks, 1, 9-length(t)));
	} else { len = length(t);
		line = line t ;
#		printf("%s", t);
	}
	nf = split(s, w);
	for (io=1; io <= nf; io++) {
		if ((len + length(w[io])) >= width) {
			line = line "\n         "; len = 9;
#			printf("\n%9s", ""); len = 9;
		}
		line = line " " w[io];
#		printf(" %s", w[io]);
		len += 1 + length(w[io]);
	}
	line = line "\n";
#	printf("\n");
}

function out1() {	# Output Header
	if (I == "")	return;
	if (sep != 0)	return;
#	print sepl;
	if (t != "") oline(I Q, t); else { oline(I Q, T); T = ""; }
	if (id>0)    oline("yCat:", id);
	if ((Y != "") || (Z != "")) oline("Status:", Y Z);
	if (S != "") oline("Size:", S "; " V);
	else if (Q == "") oline("Size:", V);
	if (D != "") oline("Date:", D);
	if (U != "") oline("Acronym:", U);
	if (T != "") oline("Title:", T);
	sep = 1; field = "";
}
function clearcat(){	# Reset to zero
    nL = 0;
    I=""; T=""; V=""; S=""; Y=""; D=""; Z=""; t=""; U=""; sep = 0; line = "";
    cite=""; id=0;
    for (j in aL) delete aL[j];
    for (j in aT) delete aT[j];
}
function prcat() {	# Output a complete Catalogue Description
	#print "### prcat called ###";
	if (empty_line>0) { print ""; empty_line = 0 }
	out1();
	#print "nL = ", nL;
	for (j = 1; j <= nL; j++) oline(aT[j], aL[j]);
	if (cite!="") oline(title["%q"], cite);
	print line sepl;
	clearcat();
	#print "### prcat RETURN ###";
}
#{ printf "(I=%s)%s\n", I, $0 }
/^%I/ {
    I = $2; Q = "";
    ni = split(I, aI, "=");
    if (ni > 1) {
	I = aI[1];
	if (length(I) < 18) Q = substr(blanks, 1, 18-length(I));
	ir = index(aI[2], "/");
	Q = Q " ===>" substr(blanks, 1, 6-ir) aI[2] ;
    }
    next
}
/^#  *[(]/ { if (I == "") print; else last_line = $0; next }
/^%id/ { id = $2; next }
/^%J=/ { additem(substr($0,1,3), amp(substr($0,4))); next }
/^%t/ { t = substr($0, 4); yr=$NF+0; next }
/^%A/ { ref=I; if(substr(I,length(I),1)=="/") ref=substr(ref,1,length(I)-1);
        cite = sprintf(" %d (%s)", yr, ref);
	nA =split(substr($0,4), A, ",");
	if(nA==1) cite = last_name(A[1]) cite;
	else if (nA==2) cite = last_name(A[1]) " & " last_name(A[2]) cite;
	else cite = last_name(A[1]) " et al." cite;
      }
/^%T/ { T = substr($0, 4); next }
/^%U/ { if (U != "") U = U "; " ; U = U substr($0, 4); next }
#/^%g/ { U = U " {" substr($0,4) "}" }
/^%V/ { if (V != "") V = V " + "; V = V substr($0,4); next ;}
/^%S/ { S = substr($0, 4); next }
/^%Y/ { Y = substr($0, 4); next }
/^%D/ { if (D == "") {
            i=index($0, "="); if (i<4) i=length; D = substr($0, 4, i-4)
            if(index($NF, "v=JD")>0) {
	       i=split($NF,a,"="); D = D " (VizieR: " a[i] ")"
	    }
	}
	if (index($0,"OBSOL") != 0) D = D " =OBSOLETED="
        #if ((rmo == 1) && (index(D, "OBSOL") != 0))  I = "";
        next }
/^%Z/ { if ($2 == 1) Z = "; connected to Simbad";
	if ($2 == 2) Z = "; fully Integrated in Simbad";
	next;
}
/^%Q/ { gsub(/ %Q /, "; ") }
/^%/  { additem($1, substr($0, 4)); next; }
/^$/  { #print "nL = ", nL;
	#print aT[nL];
	#print aL[nL];
	#for (i in aT) ;	# Crash otherwise !?!?!?!?!?!:q
	#print "nL=" nL aT[nL] "/" aL[nL]
	#print "#$, I=" I;
	if (I != "") prcat();
	else empty_line++;
	clearcat();
	next }
      { nL++; aT[nL] = ""; aL[nL] = $0; }
END   { if (I != "") prcat(); if (last_line != "") print last_line }
